
#include "pmacros.h"
#include "system.h"

#include <stdio.h>
#include <sched.h>
#include <stdlib.h>
#include <string.h>
#include <semaphore.h>
#include <errno.h>

#include "random.h"
#include "returncode.h"


/*  The shared buffer 
*/
#define NUMBER_OF_POSITIONS 4
int buff[NUMBER_OF_POSITIONS];
int index_producer=0;
int index_consumer=0;


/* Semaphores to manage the
   the number of empty/full positions
*/
sem_t number_of_empty_positions;
sem_t number_of_full_positions;


void* consumer(void* arg)
{
 struct timespec delay;
 int status;

 while(1)
   {
   status=sem_wait(&number_of_full_positions);
   if(status!=0)
	returncode("sem_wait full",status);

   printf("consumer = %d\n",buff[index_consumer]);
   index_consumer=(index_consumer+1)%NUMBER_OF_POSITIONS;

   status=sem_post(&number_of_empty_positions);
   if(status!=0)
	returncode("sem_post empty",status);

   delay.tv_nsec=0;
   delay.tv_sec=get_random(2);
   nanosleep(&delay,NULL);
   }
}



void* producer(void* arg)
{
 int val;
 struct timespec delay;
 int status;

 while(1)
   {
   status=sem_wait(&number_of_empty_positions);
   if(status!=0)
	returncode("sem_wait empty",status);

   val=get_random(100);
   printf("producer = %d\n",val);

   buff[index_producer]=val;
   index_producer=(index_producer+1)%NUMBER_OF_POSITIONS;

   status=sem_post(&number_of_full_positions);
   if(status!=0)
	returncode("sem_post full",status);

   delay.tv_nsec=0;
   delay.tv_sec=get_random(2);
   nanosleep(&delay,NULL);
   }


  pthread_exit(NULL);
  return 0;
}


void* POSIX_Init(void *argument)
{

 pthread_t id1, id2;
 int status;

 set_seed();

 status=sem_init(&number_of_empty_positions,0,NUMBER_OF_POSITIONS);
 if(status!=0)
	returncode("sem_init empty",status);
 status=sem_init(&number_of_full_positions,0,0);
 if(status!=0)
	returncode("sem_init full",status);

 /* create thread */
 status=pthread_create(&id1,NULL,producer,NULL);
 if(status<0)
	returncode("pthread_create producer",status);
 status=pthread_create(&id2,NULL,consumer,NULL);
 if(status<0)
	returncode("pthread_create consumer",status);
 
  pthread_exit(0);
  return 0; 
}



